<?php

namespace CreoCode\ContaoTourismBundle\Classes;

use Contao\Frontend;
use Contao\Date;
use Contao\PageModel;
use Contao\Config;
use Contao\Environment;
use Contao\StringUtil;

use CreoCode\ContaoTourismBundle\Models\OffersModel;
use CreoCode\ContaoTourismBundle\Models\OffersSetModel;

/**
 * Provide methods regarding offers sets.
 */
class Offers extends Frontend
{

	/**
	 * URL cache array
	 * @var array
	 */
	private static $arrUrlCache = array();

	/**
	 * Add offers items to the indexer
	 *
	 * @param array   $arrPages
	 * @param integer $intRoot
	 * @param boolean $blnIsSitemap
	 *
	 * @return array
	 */
	public function getSearchablePages($arrPages, $intRoot=0, $blnIsSitemap=false)
	{
		$arrRoot = array();

		if ($intRoot > 0)
		{
			$arrRoot = $this->Database->getChildRecords($intRoot, 'tl_page');
		}

		$arrProcessed = array();
		$time = Date::floorToMinute();

		// Get all offers sets
		$objSet = OffersSetModel::findByProtected('');

		// Walk through each set
		if ($objSet !== null)
		{
			while ($objSet->next())
			{
				// Skip offers sets without target page
				if (!$objSet->jumpTo)
				{
					continue;
				}

				// Skip offers sets outside the root nodes
				if (!empty($arrRoot) && !\in_array($objSet->jumpTo, $arrRoot))
				{
					continue;
				}

				// Get the URL of the jumpTo page
				if (!isset($arrProcessed[$objSet->jumpTo]))
				{
					$objParent = PageModel::findWithDetails($objSet->jumpTo);

					// The target page does not exist
					if ($objParent === null)
					{
						continue;
					}

					// The target page has not been published (see #5520)
					if (!$objParent->published || ($objParent->start != '' && $objParent->start > $time) || ($objParent->stop != '' && $objParent->stop <= ($time + 60)))
					{
						continue;
					}

					if ($blnIsSitemap)
					{
						// The target page is protected (see #8416)
						if ($objParent->protected)
						{
							continue;
						}

						// The target page is exempt from the sitemap (see #6418)
						if ($objParent->sitemap == 'map_never')
						{
							continue;
						}
					}

					// Generate the URL
					$arrProcessed[$objSet->jumpTo] = $objParent->getAbsoluteUrl(Config::get('useAutoItem') ? '/%s' : '/items/%s');
				}

				$strUrl = $arrProcessed[$objSet->jumpTo];

				// Get the items
				$objArticle = OffersModel::findPublishedDefaultByPid($objSet->id);

				if ($objArticle !== null)
				{
					while ($objArticle->next())
					{
						$arrPages[] = $this->getLink($objArticle, $strUrl);
					}
				}
			}
		}

		return $arrPages;
	}

	/**
	 * Generate a URL and return it as string
	 *
	 * @param OffersModel $objItem
	 * @param boolean   $blnAddSet
	 * @param boolean   $blnAbsolute
	 *
	 * @return string
	 */
	public static function generateOffersUrl($objItem, $blnAddArchive=false, $blnAbsolute=false)
	{
		$strCacheKey = 'id_' . $objItem->id . ($blnAbsolute ? '_absolute' : '');

		// Load the URL from cache
		if (isset(self::$arrUrlCache[$strCacheKey]))
		{
			return self::$arrUrlCache[$strCacheKey];
		}

		// Initialize the cache
		self::$arrUrlCache[$strCacheKey] = null;

		if (($objTarget = $objItem->getRelated('jumpTo')) instanceof PageModel)
		{
			/** @var PageModel $objTarget */
			self::$arrUrlCache[$strCacheKey] = StringUtil::ampersand($blnAbsolute ? $objTarget->getAbsoluteUrl() : $objTarget->getFrontendUrl());
		}

		// Link to the default page
		if (self::$arrUrlCache[$strCacheKey] === null)
		{
			$objPage = PageModel::findByPk($objItem->getRelated('pid')->jumpTo);

			if (!$objPage instanceof PageModel)
			{
				self::$arrUrlCache[$strCacheKey] = StringUtil::ampersand(Environment::get('requestUri'));
			}
			else
			{
				$params = '/' . ($objItem->alias ?: $objItem->id);

				self::$arrUrlCache[$strCacheKey] = StringUtil::ampersand($blnAbsolute ? $objPage->getAbsoluteUrl($params) : $objPage->getFrontendUrl($params));
			}
		}

		return self::$arrUrlCache[$strCacheKey];
	}

	/**
	 * Return the link of a offers article
	 *
	 * @param OffersModel $objItem
	 * @param string    $strUrl
	 * @param string    $strBase
	 *
	 * @return string
	 */
	protected function getLink($objItem, $strUrl, $strBase='')
	{
		if (($objTarget = $objItem->getRelated('jumpTo')) instanceof PageModel)
		{
			/** @var PageModel $objTarget */
			return $objTarget->getAbsoluteUrl();
		}

		// Backwards compatibility (see #8329)
		if ($strBase != '' && !preg_match('#^https?://#', $strUrl))
		{
			$strUrl = $strBase . $strUrl;
		}

		// Link to the default page
		return sprintf(preg_replace('/%(?!s)/', '%%', $strUrl), ($objItem->alias ?: $objItem->id));
	}

}

class_alias(Offers::class, 'Offers');
