<?php

namespace CreoCode\ContaoTourismBundle\Modules;

use Contao\Model\Collection;
use FOS\HttpCache\ResponseTagger;

use Contao\Module;
use Contao\FrontendUser;
use Contao\FrontendTemplate;
use Contao\StringUtil;
use Contao\System;
use Contao\ContentModel;
use Contao\FilesModel;

use CreoCode\ContaoTourismBundle\Classes\Accommodations;
use CreoCode\ContaoTourismBundle\Models\AccommodationsSetModel;

/**
 * Parent class for accommodations modules.
 *
 * @property string $accommodations_template
 * @property mixed  $accommodations_metaFields
 *
 */
abstract class ModuleAccommodations extends Module
{

	/**
	 * Sort out protected sets
	 *
	 * @param array $arrSets
	 *
	 * @return array
	 */
	protected function sortOutProtected($arrSets)
	{
		if (empty($arrSets) || !\is_array($arrSets))
		{
			return $arrSets;
		}

		$this->import(FrontendUser::class, 'User');
		$objSet = AccommodationsSetModel::findMultipleByIds($arrSets);
		$arrSets = array();

		if ($objSet !== null)
		{
			while ($objSet->next())
			{
				if ($objSet->protected)
				{
					if (!FE_USER_LOGGED_IN || !\is_array($this->User->groups))
					{
						continue;
					}

					$groups = StringUtil::deserialize($objSet->groups);

					if (empty($groups) || !\is_array($groups) || !\count(array_intersect($groups, $this->User->groups)))
					{
						continue;
					}
				}

				$arrSets[] = $objSet->id;
			}
		}

		return $arrSets;
	}

	/**
	 * Parse an item and return it as string
	 *
	 * @param AccommodationsModel $objArticle
	 * @param boolean   $blnAddSet
	 * @param string    $strClass
	 * @param integer   $intCount
	 *
	 * @return string
	 */
	protected function parseArticle($objArticle, $blnAddSet=false, $strClass='', $intCount=0)
	{
		$objTemplate = new FrontendTemplate($this->accommodations_template);
		$objTemplate->setData($objArticle->row());

		if ($objArticle->cssClass != '')
		{
			$strClass = ' ' . $objArticle->cssClass . $strClass;
		}

		if ($objArticle->featured)
		{
			$strClass = ' featured' . $strClass;
		}

		$objTemplate->class = $strClass;
		$objTemplate->accommodationsHeadline = $objArticle->headline;
		$objTemplate->subHeadline = $objArticle->subheadline;
		$objTemplate->hasSubHeadline = $objArticle->subheadline ? true : false;
		$objTemplate->linkHeadline = $this->generateLink($objArticle->headline, $objArticle, $blnAddSet);
		$objTemplate->more = $this->generateLink($GLOBALS['TL_LANG']['MSC']['more'], $objArticle, $blnAddSet, true);
		$objTemplate->link = Accommodations::generateAccommodationsUrl($objArticle, $blnAddSet);
		$objTemplate->set = $objArticle->getRelated('pid');
		$objTemplate->count = $intCount; // see #5708
		$objTemplate->text = '';
		$objTemplate->currency = $this->currency;
		$objTemplate->currencySymbol = $this->convertCurrencyToSymbol($this->currency);
		$objTemplate->hasText = false;
		$objTemplate->hasTeaser = false;

		// Clean the RTE output
		if ($objArticle->teaser != '')
		{
			$objTemplate->hasTeaser = true;
			$objTemplate->teaser = $objArticle->teaser;
			$objTemplate->teaser = StringUtil::encodeEmail($objTemplate->teaser);
		}

		// Display the "read more" button for external/article links
		if ($objArticle->source != 'default')
		{
			$objTemplate->text = true;
			$objTemplate->hasText = true;
		}

		// Compile the accommodations text
		else
		{
			$id = $objArticle->id;

			$objTemplate->text = function () use ($id)
			{
				$strText = '';
				$objElement = ContentModel::findPublishedByPidAndTable($id, 'tl_accommodations');

				if ($objElement !== null)
				{
					while ($objElement->next())
					{
						$strText .= $this->getContentElement($objElement->current());
					}
				}

				return $strText;
			};

			$objTemplate->hasText = function () use ($objArticle)
			{
				return ContentModel::countPublishedByPidAndTable($objArticle->id, 'tl_accommodations') > 0;
			};
		}

		$objTemplate->addImage = false;

		// Add an image
		if ($objArticle->singleSRC != '')
		{
			$imgSize = $objArticle->size ?: null;

			// Override the default image size
			if ($this->imgSize)
			{
				$size = StringUtil::deserialize($this->imgSize);

				if ($size[0] > 0 || $size[1] > 0 || is_numeric($size[2]) || ($size[2][0] ?? null) === '_')
				{
					$imgSize = $this->imgSize;
				}
			}

			$figureBuilder = System::getContainer()
				->get('contao.image.studio')
				->createFigureBuilder()
				->from($objArticle->singleSRC)
				->setSize($imgSize);

			// If the external link is opened in a new window, open the image link in a new window as well (see #210)
			if ('external' === $objTemplate->source && $objTemplate->target)
			{
				$figureBuilder->setLinkAttribute('target', '_blank');
			}

			if (null !== ($figure = $figureBuilder->buildIfResourceExists()))
			{
				// Rebuild with link to the news article if we are not in a
				// newsreader and there is no link yet. $intCount will only be
				// set by the news list and news archive modules (see #5851).
				if ($intCount > 0 && !$figure->getLinkHref())
				{
					$linkTitle = StringUtil::specialchars(sprintf($GLOBALS['TL_LANG']['MSC']['readMore'], $objArticle->headline), true);

					$figure = $figureBuilder
						->setLinkHref($objTemplate->link)
						->setLinkAttribute('title', $linkTitle)
						->build();
				}

				$figure->applyLegacyTemplateData($objTemplate, null, $objArticle->floating);
			}
		}

        // Add multi src
		$objTemplate->multiSRC = StringUtil::deserialize($objArticle->multiSRC);

		// HOOK: add custom logic
		if (isset($GLOBALS['TL_HOOKS']['parseArticles']) && \is_array($GLOBALS['TL_HOOKS']['parseArticles']))
		{
			foreach ($GLOBALS['TL_HOOKS']['parseArticles'] as $callback)
			{
				$this->import($callback[0]);
				$this->{$callback[0]}->{$callback[1]}($objTemplate, $objArticle->row(), $this);
			}
		}

		// Tag the response
		if (System::getContainer()->has('fos_http_cache.http.symfony_response_tagger'))
		{
			/** @var ResponseTagger $responseTagger */
			$responseTagger = System::getContainer()->get('fos_http_cache.http.symfony_response_tagger');
			$responseTagger->addTags(array('contao.db.tl_accommodations.' . $objArticle->id));
			$responseTagger->addTags(array('contao.db.tl_accommodations_set.' . $objArticle->pid));
		}

		return $objTemplate->parse();
	}

	/**
	 * Parse one or more items and return them as array
	 *
	 * @param Collection $objArticles
	 * @param boolean    $blnAddSet
	 *
	 * @return array
	 */
	protected function parseArticles($objArticles, $blnAddSet=false)
	{
		$limit = $objArticles->count();

		if ($limit < 1)
		{
			return array();
		}

		$count = 0;
		$arrArticles = array();

		while ($objArticles->next())
		{
			/** @var AccommodationsModel $objArticle */
			$objArticle = $objArticles->current();

			$arrArticles[] = $this->parseArticle($objArticle, $blnAddSet, ((++$count == 1) ? ' first' : '') . (($count == $limit) ? ' last' : '') . ((($count % 2) == 0) ? ' odd' : ' even'), $count);
		}

		return $arrArticles;
	}

	/**
	 * Generate a URL and return it as string
	 *
	 * @param AccommodationsModel $objItem
	 * @param boolean   $blnAddSet
	 *
	 * @return string
	 *
	 * @deprecated Deprecated since Contao 4.1, to be removed in Contao 5.
	 *             Use Accommodations::generateAccommodationsUrl() instead.
	 */
	protected function generateAccommodationsUrl($objItem, $blnAddSet=false)
	{
		@trigger_error('Using ModuleAccommodations::generateAccommodationsUrl() has been deprecated and will no longer work in Contao 5.0. Use Accommodations::generateAccommodationsUrl() instead.', E_USER_DEPRECATED);

		return Accommodations::generateAccommodationsUrl($objItem, $blnAddSet);
	}

	/**
	 * Generate a link and return it as string
	 *
	 * @param string    $strLink
	 * @param AccommodationsModel $objArticle
	 * @param boolean   $blnAddSet
	 * @param boolean   $blnIsReadMore
	 *
	 * @return string
	 */
	protected function generateLink($strLink, $objArticle, $blnAddSet=false, $blnIsReadMore=false)
	{
		// Internal link
		if ($objArticle->source != 'external')
		{
			return sprintf('<a href="%s" title="%s" itemprop="url"><span itemprop="headline">%s</span>%s</a>',
							Accommodations::generateAccommodationsUrl($objArticle, $blnAddSet),
							StringUtil::specialchars(sprintf($GLOBALS['TL_LANG']['MSC']['readMore'], $objArticle->headline), true),
							$strLink,
							($blnIsReadMore ? '<span class="invisible"> '.$objArticle->headline.'</span>' : ''));
		}

		// Encode e-mail addresses
		if (substr($objArticle->url, 0, 7) == 'mailto:')
		{
			$strArticleUrl = StringUtil::encodeEmail($objArticle->url);
		}

		// Ampersand URIs
		else
		{
			$strArticleUrl = ampersand($objArticle->url);
		}

		// External link
		return sprintf('<a href="%s" title="%s"%s itemprop="url"><span itemprop="headline">%s</span></a>',
						$strArticleUrl,
						StringUtil::specialchars(sprintf($GLOBALS['TL_LANG']['MSC']['open'], $strArticleUrl)),
						($objArticle->target ? ' target="_blank"' : ''),
						$strLink);
	}

	/**
	 * Converts currency to symbol
	 *
	 * @return string
	 */
	protected function convertCurrencyToSymbol($currency)
	{
		$currencySymbol = $GLOBALS['TL_LANG']['tl_tourism']['currency_symbol']['EUR'];
		return $currencySymbol;
	}
}

class_alias(ModuleAccommodations::class, 'ModuleAccommodations');
