<?php

namespace CreoCode\ContaoTourismBundle\Modules;

use Contao\CoreBundle\Exception\PageNotFoundException;
use Contao\Model\Collection;
use Patchwork\Utf8;

use Contao\BackendTemplate;
use Contao\StringUtil;
use Contao\Environment;
use Contao\System;

use CreoCode\ContaoTourismBundle\Models\PricesModel;

/**
 * Front end module "prices list".
 *
 * @property array  $prices_sets
 * @property string $prices_featured
 * @property string $prices_order
 */
class ModulePricesList extends ModulePrices
{

	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'mod_priceslist';

	/**
	 * Display a wildcard in the back end
	 *
	 * @return string
	 */
	public function generate()
	{
		$request = System::getContainer()->get('request_stack')->getCurrentRequest();

		if ($request && System::getContainer()->get('contao.routing.scope_matcher')->isBackendRequest($request))
		{
			$objTemplate = new BackendTemplate('be_wildcard');
			$objTemplate->wildcard = '### ' . $GLOBALS['TL_LANG']['FMD']['priceslist'][0] . ' ###';
			$objTemplate->title = $this->headline;
			$objTemplate->id = $this->id;
			$objTemplate->href = 'contao/main.php?do=themes&amp;table=tl_module&amp;act=edit&amp;id=' . $this->id;

			return $objTemplate->parse();
		}

		$this->prices_sets = $this->sortOutProtected(StringUtil::deserialize($this->prices_sets));

		// Return if there are no sets
		if (empty($this->prices_sets) || !\is_array($this->prices_sets))
		{
			return '';
		}

		return parent::generate();
	}

	/**
	 * Generate the module
	 */
	protected function compile()
	{
		$limit = null;
		$offset = (int) $this->skipFirst;

		// Maximum number of items
		if ($this->numberOfItems > 0)
		{
			$limit = $this->numberOfItems;
		}

		// Handle featured prices
		if ($this->prices_featured == 'featured')
		{
			$blnFeatured = true;
		}
		elseif ($this->prices_featured == 'unfeatured')
		{
			$blnFeatured = false;
		}
		else
		{
			$blnFeatured = null;
		}

		$this->Template->articles = array();
		$this->Template->empty = $GLOBALS['TL_LANG']['MSC']['emptyList'];

		// Get the total number of items
		$intTotal = $this->countItems($this->prices_sets, $blnFeatured);

		if ($intTotal < 1)
		{
			return;
		}


		$total = $intTotal - $offset;

		// Split the results
		if ($this->perPage > 0 && (!isset($limit) || $this->numberOfItems > $this->perPage))
		{
			// Adjust the overall limit
			if (isset($limit))
			{
				$total = min($limit, $total);
			}

			// Get the current page
			$id = 'page_n' . $this->id;
			$page = Input::get($id) ?? 1;

			// Do not index or cache the page if the page number is outside the range
			if ($page < 1 || $page > max(ceil($total/$this->perPage), 1))
			{
				throw new PageNotFoundException('Page not found: ' . Environment::get('uri'));
			}

			// Set limit and offset
			$limit = $this->perPage;
			$offset += (max($page, 1) - 1) * $this->perPage;
			$skip = (int) $this->skipFirst;

			// Overall limit
			if ($offset + $limit > $total + $skip)
			{
				$limit = $total + $skip - $offset;
			}

			// Add the pagination menu
			$objPagination = new Pagination($total, $this->perPage, Config::get('maxPaginationLinks'), $id);
			$this->Template->pagination = $objPagination->generate("\n  ");
		}

		$objArticles = $this->fetchItems($this->prices_sets, $blnFeatured, ($limit ?: 0), $offset);

		// Add the articles
		if ($objArticles !== null)
		{
			$this->Template->articles = $this->parseArticles($objArticles);
		}

		$this->Template->sets = $this->prices_sets;
	}

	/**
	 * Count the total matching items
	 *
	 * @param array   $pricesSets
	 * @param boolean $blnFeatured
	 *
	 * @return integer
	 */
	protected function countItems($pricesSets, $blnFeatured)
	{
		// HOOK: add custom logic
		if (isset($GLOBALS['TL_HOOKS']['pricesListCountItems']) && \is_array($GLOBALS['TL_HOOKS']['pricesListCountItems']))
		{
			foreach ($GLOBALS['TL_HOOKS']['pricesListCountItems'] as $callback)
			{
				if (($intResult = System::importStatic($callback[0])->{$callback[1]}($pricesSets, $blnFeatured, $this)) === false)
				{
					continue;
				}

				if (\is_int($intResult))
				{
					return $intResult;
				}
			}
		}

		return PricesModel::countPublishedByPids($pricesSets, $blnFeatured);
	}

	/**
	 * Fetch the matching items
	 *
	 * @param array   $pricesSets
	 * @param boolean $blnFeatured
	 * @param integer $limit
	 * @param integer $offset
	 *
	 * @return Collection|PricesModel|null
	 */
	protected function fetchItems($pricesSets, $blnFeatured, $limit, $offset)
	{
		// HOOK: add custom logic
		if (isset($GLOBALS['TL_HOOKS']['pricesListFetchItems']) && \is_array($GLOBALS['TL_HOOKS']['pricesListFetchItems']))
		{
			foreach ($GLOBALS['TL_HOOKS']['pricesListFetchItems'] as $callback)
			{
				if (($objCollection = System::importStatic($callback[0])->{$callback[1]}($pricesSets, $blnFeatured, $limit, $offset, $this)) === false)
				{
					continue;
				}

				if ($objCollection === null || $objCollection instanceof Collection)
				{
					return $objCollection;
				}
			}
		}

		// Determine sorting
		$t = PricesModel::getTable();
		$arrOptions = array();

		switch ($this->prices_order)
		{
			case 'order_sorting_asc':
				$arrOptions['order'] = "$t.sorting";
				break;

			case 'order_sorting_desc':
				$arrOptions['order'] = "$t.sorting DESC";
				break;
				
			case 'order_headline_asc':
				$arrOptions['order'] = "$t.headline";
				break;

			case 'order_headline_desc':
				$arrOptions['order'] = "$t.headline DESC";
				break;

			case 'order_random':
				$arrOptions['order'] = "RAND()";
				break;

			default:
				$arrOptions['order'] = "$t.sorting";
		}

		return PricesModel::findPublishedByPids($pricesSets, $blnFeatured, $limit, $offset, $arrOptions);
	}
}

class_alias(ModulePricesList::class, 'ModulePricesList');
